#!/usr/bin/env python3
import subprocess
import sys
import logging
import importlib.util
import inspect
import os


logger = logging.getLogger(__name__)


def pad_output(s, pad, cols=80, file=sys.stderr):
    pad_len = cols - len(s)
    left_len = int(pad_len / 2)
    right_len = pad_len - left_len
    print("{}{}{}".format(pad*left_len, s, pad*right_len), file=file)


def subprocess_run(*args, **kwargs):
    logger.info("Executing {} {}".format(" ".join(args[0]), kwargs))
    return subprocess.run(*args, **kwargs)


def import_by_path(module_name, file_path):
    spec = importlib.util.spec_from_file_location(module_name, file_path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module


def drivers_in_module(module):
    from menthol import Driver
    for name, val in inspect.getmembers(module):
        if inspect.isclass(val) and issubclass(val, Driver) and val != Driver:
            yield val


def sanity_check():
    return {
        "uname": os.uname(),
        "cpu_count": os.cpu_count()
    }



#!/usr/bin/env python3


class Configuration(object):
    """Configuration describes how a benchmark should be run.
    It selects the implementation (for example, different implementations) and
    implementation-specific options (for example, whether replay compilation
    should be used). It also derives options from current Run instance, which is
    shared by all configurations (for example, relative heap size, the number of
    threads, etc.)
    """

    def __init__(self, name):
        self.name = name
        self.args = {}
    
    def update_args(self, args):
        self.args.update(args)

    def build(self, benchmark):
        raise NotImplementedError

    def get_cmd(self, benchmark):
        raise NotImplementedError

    def clean(self, benchmark):
        raise NotImplementedError



#!/usr/env/bin python3
import logging
import argparse

import sys
from pathlib import Path

from menthol import __VERSION__
from menthol.util import import_by_path, drivers_in_module

logger = logging.getLogger(__name__)

def setup_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", action="store_true",
                        help="change logging level to DEBUG")
    parser.add_argument("-i", "--invocation", type=int, default=20,
                        help="how many invocation")
    parser.add_argument("--version", action="version",
                        version="menthol {}".format(__VERSION__))
    parser.add_argument("FILE",
                        help="path to discover drivers")
    subparsers = parser.add_subparsers()
    clean = subparsers.add_parser("clean")
    clean.set_defaults(which="clean")
    return parser


def main():
    parsers = setup_parser()
    args = vars(parsers.parse_args())

    # Config root logger
    if args.get("verbose") == True:
        log_level = logging.DEBUG
    else:
        log_level = logging.INFO
    logging.basicConfig(
        format="[%(levelname)s] %(asctime)s %(filename)s:%(lineno)d %(message)s",
        level=log_level)

    file_path = args["FILE"]
    if not Path(file_path).is_file():
        logger.critical("Failed to load {}. No such file.".format(file_path))
        sys.exit(1)
    mod = import_by_path("custom_driver", file_path)
    logger.info("{} loaded".format(file_path))

    # Handle subcommands
    if not args.get("which"):
        # Default subcommand, which is benchmarking
        for driver_cls in drivers_in_module(mod):
            driver = driver_cls()
            driver.set_invocation(args["invocation"])
            driver.build()
            driver.start()
    elif args.get("which") == "clean":
        # Cleanings
        for driver_cls in drivers_in_module(mod):
            driver = driver_cls()
            driver.clean()


if __name__ == "__main__":
    main()



import datetime
import socket
import os
import json
import sys
import logging
import subprocess
from collections import defaultdict

from menthol.util import subprocess_run, sanity_check

logger = logging.getLogger(__name__)


def run_with_args(exe, args_mapping, driver_args, config_args, bm_args):
    cmd = [exe]
    # Config args will override driver args
    args = {**driver_args, **bm_args, **config_args,}
    # Benchmark args shouldn't conflicts with config args
    for k in bm_args:
        if k in config_args:
            logger.error(("Argument {} appears both in "
                          "config args: {} "
                          "and benchmark args: {}"
                          ).format(k, config_args[k], bm_args[k]))
            sys.exit(1)
    optional_args = filter(lambda x:not isinstance(x, int),
                           args_mapping.keys())
    positional_args = sorted(filter(lambda x:isinstance(x, int),
                                    args_mapping.keys()))

    for arg in optional_args:
        cmd.append("--{}={}".format(arg, args[args_mapping[arg]]))
    for arg in positional_args:
        cmd.append("{}".format(args[args_mapping[arg]])) 

    result = subprocess_run(cmd,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    stdout = result.stdout.decode("utf-8")
    stderr = result.stderr.decode("utf-8")
    logger.debug("stdout:\n{}".format(stdout))
    logger.debug("stderr:\n{}".format(stderr))
    return cmd, stdout, stderr


class Driver(object):
    def __init__(self, log_dir):
        self.log_dir = log_dir
        self.benchmarks = []
        self.configurations = []
        self.args = {}
        self.results = []

    def set_invocation(self, invocation):
        self.invocation = invocation
        logger.info("Set invocation to {}".format(self.invocation))

    def add_benchmark(self, benchmark):
        self.benchmarks.append(benchmark)
        benchmark.set_driver(self)

    def add_configuration(self, configuration):
        self.configurations.append(configuration)

    def update_args(self, args):
        self.args.update(args)

    def clean(self):
        for benchmark in self.benchmarks:
            for configuration in self.configurations:
                configuration.clean(benchmark)

    def build(self):
        for benchmark in self.benchmarks:
            for configuration in self.configurations:
                configuration.build(benchmark)

    def start(self):
        if not getattr(self, "invocation"):
            logger.critical("Invocation not set")
            sys.exit(1)
        self.begin()
        self.end()
        while not self.should_stop():
            self.begin()
            self.end()
        self.stop()

    def begin(self):
        result = {
            "driver_args": self.args.copy(),
            "benchmarks": defaultdict(list)
        }
        for bm in self.benchmarks:
            for i in range(0, self.invocation):
                logger.info("Invocation {}".format(i))
                for config in self.configurations:
                    cmd, stdout, stderr = run_with_args(
                        exe=config.get_cmd(bm),
                        args_mapping=bm.get_mapping(config),
                        driver_args=self.args,
                        config_args=config.args,
                        bm_args = bm.args)
                    stats = bm.parse(stdout, stderr)
                    log = {
                        "cmd": cmd,
                        "stdout": stdout,
                        "stderr": stderr,
                        "stats": stats,
                        "config": config.name
                    }
                    result["benchmarks"][bm.name].append(log)
        self.results.append(result)

    def end(self):
        pass

    def should_stop(self):
        return True

    def stop(self):
        date_str = datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S")
        hostname = socket.gethostname()

        log_filename = "{}-{}.json".format(hostname, date_str)
        os.makedirs(self.log_dir, exist_ok=True)
        log_path = os.path.join(self.log_dir, log_filename)

        log = sanity_check()
        log["results"] = self.results

        with open(log_path, "w") as log_file:
            json.dump(log, fp=log_file, indent=4)

        logger.info("Log dumped to {}".format(log_path))



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='menthol',
    version=__import__("menthol").__VERSION__,
    description='A library that helps you exercising good benchmarking methodology',
    long_description=long_description,
    url='https://github.com/caizixian/menthol',
    author='Zixian Cai',
    author_email='u5937495@anu.edu.au',
    license='Apache',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: System :: Benchmark',
        'Programming Language :: Python :: 3',
    ],

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    entry_points={
        'console_scripts': [
            'menthol = menthol.__main__:main'
        ]
    },

    install_requires=[],
    extras_require={},
)